.search-view menubutton button:focus:focus-visible {
    outline-color: alpha(@window_fg_color, 0.8);
}

.search-view {
    background-color: @window_bg_color;
    color: @window_fg_color;
}

.forecast-card separator {
    background-color: @borders;
}
